/* --- Copyright University of Birmingham 1997. All rights reserved. ------
 > File:            $poplocal/local/auto/get_command_output.p
 > Purpose:			Run a shell command and read in input
 > Author:          Aaron Sloman, Oct  5 1997
 > Documentation:	HELP * GET_COMMAND_OUTPUT
 > Related Files:
 */

/*
get_command_output(shell_command) -> string
get_command_output(shell_command, true) -> zero or more strings
	Given a unix command, returns a string with the output.

get_command_output(['/bin/who'])=>
[%get_command_output(['/bin/who'], true)%]==>
get_command_output(['/bham/bin/hostname'])=>
get_command_output(['/bham/bin/mailname' 'axs'])=>
get_command_output(['/bham/bin/mailname' 'xyz'])=>
[%get_command_output(['/bin/sh' '-c' 'ls -l pop*'], true)%]==>

[%get_command_output(['/usr/ucb/ps' '-uxw'], true)%]==>

*/

section;

;;; Allow at most three seconds wait
global vars get_command_timeout = 3e6;

define global procedure get_command_output(arglist, /*alloutput*/) /* -> string */ ;

	lvars all_output;
	if isboolean(arglist) then
		arglist -> (arglist, all_output);
	else
		false -> all_output
	endif;


	lvars
		pipe_device, repeater, string;

	lvars answer_found = false;

	define lconstant cleanup();
		sysclose(pipe_device);
	enddefine;

	define timeout();
		if answer_found then
			false -> answer_found
		else
			termin;
			exitfrom(repeater)
		endif;
	enddefine;
		
	if sys_file_exists(hd(arglist)) then

		;;; spawn process, and get pipe device to read in output
		pipein(hd(arglist), arglist, false) -> pipe_device;

		;;; make a string repeater from the device
		line_repeater(pipe_device, sysstring) -> repeater;

		;;; get output line
		repeat
			false -> answer_found;
			get_command_timeout-> sys_timer(timeout);
			repeater() -> string;
			true -> answer_found;
			;;; cancel timeout procedure.
			false -> sys_timer(timeout);
			if string == termin then
				;;; no more output
				cleanup();
				unless all_output then
					false /* -> string */;
				endunless;
				return()
			else
				string
			endif;
		quitunless(all_output or datalength(string) == 0);
		endrepeat;

		cleanup();
	else
		unless all_output then
			false /* -> string */;	
		endunless
	endif
enddefine;


endsection;
