/* --- Copyright University of Birmingham 2003. All rights reserved. ------
 > File:            $poplocal/local/auto/vedncdxtermkeys.p
 > Purpose:			Key bindings for NCD (and HP, and PC) X terminal
 > Author:          Aaron Sloman, Oct 13 1995 (see revisions)
 > Documentation:	HELP * NCDXTERMKEYS
 > Related Files:
 */


/* --- Copyright University of Sussex 1992. All rights reserved. ----------
 > File:			C.all/lib/ved/term/vedncdxtermkeys.p
 > Purpose:			VED: key bindings for NCD X terminal
 > Author:			Rob Smith, Sep 14 1989 (see revisions)
 > Related Files:
 */

/* This file defines the procedure -vedncdxtermkeys- which customises
	VED's key bindings for the keyboard of an NCD16 X terminal

	In order to use the keys marked "Home" and "End" in VED, the following
	key translations must be added to /usr/lib/X11/app-defaults/XTerm:

		XTerm*VT100*Translations: #override \
			<Key>Home: string(0x1b) string("<") \n\
			<Key>End: string(0x1b) string(">")

	i.e. HOME will send ESC <, and END will send ESC >
*/

section;

uses vedvt100keys;

define global vedncdxtermkeys();
	vedvt100keys();
	vedset keys
		"charup"      	= esc [ A		   	;;; UP
		"chardown"        = esc [ B		   	;;; DOWN
		"charleft"        = esc [ D		   	;;; LEFT
		"charright"         = esc [ C		   	;;; RIGHT

		"topfile"           = esc esc [ A		;;; ESC UP
		"endfile"           = esc esc [ B		;;; ESC DOWN
		"textleft"          = esc esc [ D		;;; ESC LEFT
		"screenright"       = esc esc [ C	   	;;; ESC RIGHT
/*
	   	   ;;; Copy the above for esc A, esc B, esc C, esc D
		"screenup"          = esc  A	   	    ;;; UP
		"screendown"        = esc  B	   	    ;;; DOWN
		"screenleft"        = esc  D	   	    ;;; LEFT
		"textright"         = esc  C	   	    ;;; RIGHT

		"topfile"           = esc esc  A	   	;;; ESC UP
		"endfile"           = esc esc  B	   	;;; ESC DOWN
		"textleft"          = esc esc  D	   	;;; ESC LEFT
		"screenright"       = esc esc  C	   	;;; ESC RIGHT
*/

		"marklo"            = esc [ 1 1 ~	   	;;; F1
		"marklo"            = esc O P	   		;;; F1 (on PC)
		"markhi"            = esc [ 1 2 ~	   	;;; F2
		"markhi"            = esc O Q	   		;;; F2 (on PC)
		"clearhead"         = esc [ 1 3 ~	   	;;; F3
		"clearhead"         = esc O R	   		;;; F3 (on PC)
		"linedelete"        = esc [ 1 4 ~	   	;;; F4
		"linedelete"        = esc O S	   		;;; F4 (on PC)

		"cleartail"         = esc [ 1 5 ~	   	;;; F5
		"wordleftdelete"    = esc [ 1 7 ~	   	;;; F6
		"wordrightdelete"   = esc [ 1 8 ~	    ;;; F7
		"ENTER m"           = esc [ 1 9 ~	    ;;; F8

		"ENTER t"           = esc [ 2 0 ~	    ;;; F9
		"xrefresh"          = esc [ 2 1 ~	   	;;; F10
		"pushkey"           = esc [ 2 3 ~	    ;;; F11
		"exchangeposition"  = esc [ 2 4 ~	   	;;; F12

		"ENTER mbf"         = esc esc [ 1 1 ~	;;; ESC F1
		"ENTER mbf"         = esc esc O P		;;; ESC F1 (on PC)
		"ENTER mef"         = esc esc [ 1 2 ~	;;; ESC F2
		"ENTER mef"         = esc esc O Q		;;; ESC F2 (On PC)
		"ENTER yankw"       = esc esc [ 1 3 ~	;;; ESC F3
		"ENTER yankw"       = esc esc O R		;;; ESC F3 (On PC)
		"ENTER yankl"       = esc esc [ 1 4 ~	;;; ESC F4
		"ENTER yankl"       = esc esc O S		;;; ESC F4 (On PC)

		"ENTER yankw"       = esc esc [ 1 5 ~	;;; ESC F5
		"ENTER yankw"       = esc esc [ 1 7 ~	;;; ESC F6
		"ENTER yankw"       = esc esc [ 1 8 ~	;;; ESC F7
		"ENTER mi"          = esc esc [ 1 9 ~	;;; ESC F8

		"ENTER ti"          = esc esc [ 2 0 ~	;;; ESC F9
		"ENTER mo"          = esc esc [ 2 1 ~	;;; ESC F10
		"popkey"            = esc esc [ 2 3 ~	;;; ESC F11
		"ENTER cps"         = esc esc [ 2 4 ~	;;; ESC F12

		"lineabove"         = esc [ 2 ~	   	    ;;; INSERT
		"prevscreen"        = esc [ 5 ~	   	    ;;; PAGE UP
		"nextscreen"        = esc [ 6 ~	   	    ;;; PAGE DOWN
		"topfile"           = esc <	   	      	;;; HOME
		"endfile"           = esc >	   	      	;;; END

		"linebelow"         = esc esc [ 2 ~	   	;;; ESC INSERT
		"ENTER xup"         = esc esc [ 5 ~	   	;;; ESC PAGE UP
		"ENTER xdn"         = esc esc [ 6 ~	   	;;; ESC PAGE DOWN
		"markfind"          = esc esc <	   	   	;;; ESC HOME
		"endrange"          = esc esc >	   	   	;;; ESC END

		"ENTER timed_esc"   = esc O u	   	   	;;; Keypad 5
		"helpkey"           = esc O o	   	   	;;; Keypad /
		"helpkey"           = esc [ F F A F	   	;;; Keypad /
		"loadline"          = esc O j	   	   	;;; Keypad *
		"loadline"          = esc [ F F A A	   	;;; Keypad *
		"statusswitch"      = esc O k	   	   	;;; Keypad +

		"ENTER hkey"        = esc esc O o	   	;;; ESC Keypad /
		"ENTER lmr"         = esc esc O j	   	;;; ESC Keypad *

		;;; EXTRAS FOR VERSION N-123UK
		;;; The "/" key = R5, and the "*" key = R6

		"textleft"			=   esc [ 1 1 1 ~       ;;; R1  Pause
		"screenup"			=   esc [ 1 1 2 ~       ;;; R2  PrScr
		"textright"			=   esc [ 1 1 3 ~       ;;; R3  ScrLock
		"redocommand"		=   esc [ 1 1 4 ~       ;;; R4  Pad "-"
		"screendown"		=   esc [ 1 1 5 ~       ;;; R5  "/"
		"screenright"		=   esc [ 1 1 7 ~       ;;; R6  "*"
		"charupleft"		=   esc [ 1 1 8 ~       ;;; R7  (7)
		"charup"			=   esc [ 1 1 9 ~		;;; R8  (8)
		"charupright"		=   esc [ 1 2 0 ~       ;;; R9  (9)
		"charleft"			=   esc [ 1 2 1 ~		;;; R10 (4)
		"ENTER timed_esc"	=   esc [ 1 2 3 ~       ;;; R11 (5)
		"charright"			=   esc [ 1 2 4 ~		;;; R12 (6)
		"chardownleft"		=   esc [ 1 2 5 ~       ;;; R13 (1)
		"chardown"			=   esc [ 1 2 6 ~		;;; R14 (2)
		"chardownright"		=   esc [ 1 2 8 ~       ;;; R15 (3)

		"topwindow"			=   esc esc [ 1 1 1 ~   ;;; ESC R1 Pause
		"topfile"			=   esc esc [ 1 1 2 ~   ;;; ESC R2 PrScr
		"markfind"			=   esc esc [ 1 1 3 ~   ;;; ESC R3 ScrLock
		"ENTER lmr"			=   esc esc [ 1 1 4 ~   ;;; ESC keypad "="
		"endfile"			=   esc esc [ 1 1 5 ~   ;;; ESC R5 "/" and Top Right
		"ENTER lmr"			=   esc esc [ 1 1 7 ~   ;;; ESC R6 "*"
		"charupleftlots"	=   esc esc [ 1 1 8 ~   ;;; ESC R7 (7)
		"charuplots"		=   esc esc [ 1 1 9 ~   ;;; ESC R8 (8)
		"charuprightlots"	=   esc esc [ 1 2 0 ~   ;;; ESC R9 (9)
		"charleftlots"		=   esc esc [ 1 2 1 ~	;;; ESC R1 (4)
		"midwindow"			=   esc esc [ 1 2 3 ~   ;;; ESC R1 (5)
		"charrightlots"		=   esc esc [ 1 2 4 ~	;;; ESC R1 (6)
		"chardownleftlots"	=   esc esc [ 1 2 5 ~   ;;; ESC R1 (1)
		"chardownlots"		=   esc esc [ 1 2 6 ~	;;; ESC R1 (2)
		"chardownrightlots"	=   esc esc [ 1 2 8 ~   ;;; ESC R1 (3)

	endvedset;

	;;; extras for XVED
	vedsetkey('\^[[FFB7', "vedcharupleft");
	vedsetkey('\^[[FFB8', "vedcharup");
	vedsetkey('\^[[FFB9', "vedcharupright");
	vedsetkey('\^[[FFB4', "vedcharleft");
	vedsetkey('\^[[FFB5', "ved_timed_esc");
	vedsetkey('\^[[FFB6', "vedcharright");
	vedsetkey('\^[[FFB1', "vedchardownleft");
	vedsetkey('\^[[FFB2', "vedchardown");
	vedsetkey('\^[[FFB3', "vedchardownright");

;;; Stuff for (Motif) XVed
	vedsetkey('\^[[FF8D', "vedenter");			;;; ENTER
	vedsetkey('\^[[FFBE', "vedmarklo");				;;; F1
	vedsetkey('\^[\^[[FFBE', "ved_mbf");			;;; ESC F1

;;; Middle pad (On Sun type 5 keyboard)
	vedsetkey('\^[[FF63', "vedlineabove");		;;; Insert
	vedsetkey('\^[[FF50', "vedtopfile");		;;; Home
;;;	vedsetkey('\^[\^[[FF50', "ved????");		;;; ESC Home
	vedsetkey('\^[[FF57', "vedendfile");		;;; End
;;;	vedsetkey('\^[\^[[FF57', "ved????");		;;; ESC End
	vedsetkey('\^[[FF55', "vedprevscreen");		;;; Page up
;;;	vedsetkey('\^[\^[[FF55', "ved????");		;;; ESC Page up
	vedsetkey('\^[[FF56', "vednextscreen");		;;; Page dn
;;;	vedsetkey('\^[\^[[FF56', "ved????");		;;; ESC Page dn

;;; Arrow pad
	vedsetkey('\^[[FF52', "vedcharup");		;;; Up
	vedsetkey('\^[\^[[FF52', "vedtopfile");		;;; ESC Up
	vedsetkey('\^[[FF51', "vedcharleft");		;;; Left
	vedsetkey('\^[\^[[FF51', "vedtextleft");	;;; ESC Left
	vedsetkey('\^[[FF54', "vedchardown");		;;; Down
	vedsetkey('\^[\^[[FF54', "vedendfile");		;;; ESC Down
	vedsetkey('\^[[FF53', "vedcharright");		;;; Right
	vedsetkey('\^[\^[[FF53', "vedtextright");	;;; ESC Right

	if systranslate('LINUX') then
		vedset keys
			;;; Stuff for linux console
			"marklo" = '\^[[[A'
			"markhi" = '\^[[[B'
			"clearhead" = '\^[[[C'
			"deleteline" = '\^[[[D'
			"cleartail" = '\^[[[E'
			"enter"     =  '\^[[1~'
			"topfile"   =  '\^[[2~'
			"chardelete" = '\^[[3~'
			"endfile"   =  '\^[[4~'
		endvedset;
	endif;

    vedsetkey('\^L', "vedxrefresh");
	vedsetkey('\^H', "vedchardelete");
	
	'ncdxterm' -> vedkeymapname;
enddefine;



endsection;


/* --- Revision History ---------------------------------------------------
--- Aaron Sloman, Jan 11 2003
		Made backspace chardelete by default
--- Aaron Sloman, Sep 26 2002
		Put in more options for linux+PC
--- Aaron Sloman, Jun 19 1997
	Set CTRL-L to vedxrefresh
--- Aaron Sloman, Dec 24 1996
	Extended for NCD keyboard type N-123UK
--- Aaron Sloman, Sep  9 1996
	Alas, had to make keypad and big pad arrows both move in small
	steps, for DEC Alpha
--- Aaron Sloman, Oct 1995
	Put all procedure names in quotes. Also added Motif Xved bindings.
	Removed top leve call of vedncdxtermkeys.
--- Aaron Sloman, Oct 1992
--- Aaron Sloman, Aug 19 1992
		Made More consistent with SunKeys, by changing
		F1 to F12 mappings
--- John Williams, Aug 18 1992
		Added key mappings for the HOME and END keys
--- John Williams, Oct 12 1990
		-vedkeymapname- instead of -vedkeyboardname-
--- John Williams, Oct	5 1990
		Now uses -vedhelpkey-, and sets -vedkeyboardname-
--- John Williams & Rob Duncan, Nov	 8 1989
		Function key definitions changed to be more similar to
		LIB VEDVT220KEYS
--- Rob Duncan, Oct 9 1989
		Sectionised; added call to -vedvt100keys-
--- Andreas Schoter Sep 1989
		Modified to use the -vedset- notation
 */
