/* --- Copyright University of Sussex 1994.  All rights reserved. ---------
 > File:           $poplocal/local/lib/routetree.p
 > Purpose:        Graphical presentation of search tree
 > Author:         David Young, Nov 20 1986 (See Revisions)
 > Documentation:  TEACH * ROUTE
 > Related Files:  LIB * ROUTE, LIB * SEETREE, LIB * SHOWTREE
 */

section;

uses seetree;


define isleaf(node);
	lvars node;
    false;
enddefine;

define first3(word);
	lvars word;
    word >< '   ' -> word;
    consstring(word(1),word(2),word(3),3);
enddefine;

define name_&_subtrees(place, deststatline, routelist) -> tree;
	lvars place, deststatline, subtrees, routelist, tree;

	;;; pattern variables
    vars name, statline nextstatline time nexttime word  oldstatline;

    if place matches [start ?name] then
        [start] -> statline;
    else
        place --> [?statline ?time ?oldstatline];

		if hd(statline) == hd(oldstatline) then
			;;; new station, same line, get name of station
			tl(statline)
		else
			;;; new line, get name of line
			statline
		endif -> name;
		;;; add time to abbreviated name
        first3(hd(name)) <> ' ' >< time -> name;

        if statline = deststatline then '*' <> ' ' <> name -> name endif;

    endif;
    [% name,
        foreach [arrived ?nextstatline at ?nexttime mins from ^statline]
             in routelist
		do
			 name_&_subtrees(
				[^nextstatline ^nexttime ^statline],
				deststatline,
				routelist)
        endforeach
     %] -> tree;
enddefine;

define getroutetree() -> tree;
    vars statline, deststatline;
	lvars name word;
    if present([arrived ?statline at = mins from [start]])
    and present([finished at ?deststatline]) then

    	tl(statline) -> statline;
    	dest(statline) -> statline -> name;
    	for word in statline do name >< ' ' >< word -> name endfor;

		name_&_subtrees([start ^name], deststatline, database) -> tree;

	else
        mishap('No route present in database', [])
	endif
enddefine;

define routetree();
	;;; get the route and display it using seetree
	seetree(getroutetree())
enddefine;

endsection;

/* --- Revision History ---------------------------------------------------
--- Aaron Sloman, Dec  7 1994
	Cleaned up and made to work
	Separated routetree and getroutetree
 */
